@@ -6,7 +6,7 @@ WARNING: This messes up some global state.  DO NOT import into modules
 doing regular work.  testtricks is the module for that kind for stuff.
 """
 
-#c Copyright 2008-2020, the GAVO project
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -151,6 +151,7 @@ osinter.sendMail = lambda *args, **kwargs: None
 from gavo import utils
 from gavo import svcs
 from gavo.base import config
+from gavo.base import sqlsupport
 
 
 class FakeSimbad(object):
@@ -291,7 +292,10 @@ class ForkingSubprocess(subprocess.Popen):
 							 p2cread, p2cwrite,
 							 c2pread, c2pwrite,
 							 errread, errwrite,
-							 restore_signals, start_new_session):
+							 restore_signals, 
+               # post-3.7, some additional parameters were added, which
+               # fortunately we don't need.
+							 *ignored_args):
 # stolen largely from 2.7 subprocess.  Unfortunately, I can't just override the
 # exec action; so, I'm replacing the the whole exec shebang with a simple
 # call to executable().
@@ -394,7 +398,7 @@ class ForkingSubprocess(subprocess.Popen):
 										os.write(errpipe_write, pickle.dumps(exc_value))
 										os.close(errpipe_write)
 										os._exit(255)
-								
+
 								os.close(errpipe_write)
 								try:
 									executable()
@@ -536,7 +540,6 @@ class VerboseTest(testresources.ResourcedTestCase):
 		retcode = p.wait()
 
 		try:
-			self.assertEqual(expectedRetcode, retcode)
 			
 			if isinstance(expectedStderr, (bytes, str)):
 				self.assertEqual(err, utils.bytify(expectedStderr))
@@ -547,6 +550,8 @@ class VerboseTest(testresources.ResourcedTestCase):
 			else:
 				self.assertTrue(expectedStderr(err), "Stderr didn't match functional"
 					" expectation: %s"%err.decode("ascii", "?"))
+
+			self.assertEqual(expectedRetcode, retcode)
 		except AssertionError:
 			with open("output.stdout", "wb") as f:
 				f.write(out)
@@ -781,7 +786,17 @@ def getServerInThread(data, onlyOnce=False, contentType="text/plain"):
 	class Handler(http.server.BaseHTTPRequestHandler):
 		def log_request(self, *args):
 			pass
+
+		def do_404(self):
+			self.send_response(404, "Not Found")
+			self.send_header("Content-Type", "text/plain")
+			self.end_headers()
+			self.wfile.write(b"You know this wouldn't be here")
+
 		def do_GET(self):
+			if self.path.endswith("does-not-exist"):
+				return self.do_404()
+
 			self.send_response(200, "Ok")
 			self.send_header("Content-Type", contentType)
 			self.end_headers()
@@ -810,10 +825,11 @@ def DataServer(data):
 	"""
 	httpd, t, baseURL = getServerInThread(data)
 
-	yield baseURL
-
-	httpd.shutdown()
-	t.join(10)
+	try:
+		yield baseURL
+	finally:
+		httpd.shutdown()
+		t.join(10)
 
 
 @contextlib.contextmanager
@@ -853,7 +869,37 @@ def tempConfig(*items):
 		yield
 	finally:
 		for sect, key, val in origValues:
-			config.set(sect, key, val)
+			config.getitem(sect, key).value = val
+
+
+@contextlib.contextmanager
+def forceConnection(conn):
+	"""makes the sqlsupport connection context managers return conn
+	in the controlled block.
+
+	It is up to the programmer to make sure that actually makes sense
+	in a given test.  As opposed to the originals, there will be no
+	automatic commits here.  Caveat emptor.
+	"""
+	symbols = ["getUntrustedConn", "getTableConn", "getAdminConn",
+		"getWritableUntrustedConn", "getWritableTableConn", "getWritableAdminConn"]
+
+	def _():
+		yield conn
+	cm = contextlib.contextmanager(_)
+
+	originals = []
+	for sym in symbols:
+		originals.append(getattr(sqlsupport, sym))
+		setattr(sqlsupport, sym, cm)
+	
+	try:
+		yield
+	finally:
+		originals.reverse()
+		for sym in symbols:
+			originals.append(getattr(sqlsupport, sym))
+			setattr(sqlsupport, sym, originals.pop())
 
 
 def makeRequestArgs(aDict=None, **kwargs):
